@@ -27,10 +27,10 @@
 
 #if defined (HAVE_CAIRO_ATSUI)
 #  include "pangocairo-atsui.h"
-#elif defined (HAVE_CAIRO_FREETYPE)
-#  include "pangocairo-fc.h"
 #elif defined (HAVE_CAIRO_WIN32)
 #  include "pangocairo-win32.h"
+#elif defined (HAVE_CAIRO_FREETYPE)
+#  include "pangocairo-fc.h"
 #endif
 
 GType
@@ -78,8 +78,8 @@ pango_cairo_font_map_get_type (void)
  * You generally should only use the #PangoFontMap and
  * #PangoCairoFontMap interfaces on the returned object.
  * 
- * Return value: the newly created fontmap object. Free
- * with g_object_unref().
+ * Return value: the newly allocated #PangoFontMap, which should
+ *               be freed with g_object_unref().
  *
  * Since: 1.10
  **/
@@ -219,7 +219,7 @@ free_context_info (PangoCairoContextInfo *info)
   if (info->merged_options)
     cairo_font_options_destroy (info->merged_options);
   
-  g_free (info);
+  g_slice_free (PangoCairoContextInfo, info);
 }
 
 static PangoCairoContextInfo *
@@ -236,7 +236,7 @@ get_context_info (PangoContext *context,
 
   if (!info && create)
     {
-      info = g_new (PangoCairoContextInfo, 1);
+      info = g_slice_new (PangoCairoContextInfo);
       info->dpi = -1.0;
       info->set_options = NULL;
       info->surface_options = NULL;
@@ -386,7 +386,7 @@ pango_cairo_context_set_font_options (PangoContext               *context,
 }
   
 /**
- * pango_cairo_get_font_options:
+ * pango_cairo_context_get_font_options:
  * @context: a #PangoContext, from pango_cairo_font_map_create_context()
  * 
  * Retrieves any font rendering options previously set with
@@ -397,7 +397,7 @@ pango_cairo_context_set_font_options (PangoContext               *context,
  *   if no options have been set. This value is owned by the context
  *   and must not be modified or freed.
  **/
-const cairo_font_options_t *
+G_CONST_RETURN cairo_font_options_t *
 pango_cairo_context_get_font_options (PangoContext *context)
 {
   PangoCairoContextInfo *info;
@@ -423,7 +423,7 @@ pango_cairo_context_get_font_options (PangoContext *context)
  * Return value: the combined set of font options. This value is owned
  * by the context and must not be modified or freed.
  **/
-const cairo_font_options_t *
+G_CONST_RETURN cairo_font_options_t *
 _pango_cairo_context_get_merged_font_options (PangoContext *context)
 {
   PangoCairoContextInfo *info = get_context_info (context, TRUE);
