@@ -82,6 +82,7 @@ pango_fc_font_class_init (PangoFcFontClass *class)
 
   class->has_char = pango_fc_font_real_has_char;
   class->get_glyph = pango_fc_font_real_get_glyph;
+  class->get_unknown_glyph = NULL;
   
   object_class->finalize = pango_fc_font_finalize;
   object_class->set_property = pango_fc_font_set_property;
@@ -112,7 +113,7 @@ static void
 free_metrics_info (PangoFcMetricsInfo *info)
 {
   pango_font_metrics_unref (info->metrics);
-  g_free (info);
+  g_slice_free (PangoFcMetricsInfo, info);
 }
 
 static void
@@ -290,6 +291,17 @@ get_face_metrics (PangoFcFont      *fcfont,
   TT_OS2 *os2;
   gboolean have_transform = FALSE;
 
+  if (!face)
+    {
+      metrics->descent = 0;
+      metrics->ascent = PANGO_SCALE * PANGO_UNKNOWN_GLYPH_HEIGHT;
+      metrics->underline_thickness = PANGO_SCALE;
+      metrics->underline_position = - PANGO_SCALE;
+      metrics->strikethrough_thickness = PANGO_SCALE;
+      metrics->strikethrough_position = PANGO_SCALE * (PANGO_UNKNOWN_GLYPH_HEIGHT/2);
+      return;
+    }
+
   if  (FcPatternGetMatrix (fcfont->font_pattern,
 			   FC_MATRIX, 0, &fc_matrix) == FcResultMatch)
     {
@@ -434,7 +446,7 @@ pango_fc_font_create_metrics_for_context (PangoFcFont   *fcfont,
   g_object_unref (layout);
 
   return metrics;
-}		    
+}
 
 /* This function is cut-and-pasted into pangocairo-fcfont.c - it might be
  * better to add a virtual fcfont->create_context (font).
@@ -467,8 +479,8 @@ pango_fc_font_get_metrics (PangoFont     *font,
       if (!fcfont->fontmap)
 	return pango_font_metrics_new ();
 
-      info = g_new0 (PangoFcMetricsInfo, 1);
-      
+      info = g_slice_new0 (PangoFcMetricsInfo);
+
       fcfont->metrics_by_lang = g_slist_prepend (fcfont->metrics_by_lang, 
 						 info);
 	
@@ -532,7 +544,7 @@ pango_fc_font_real_get_glyph (PangoFcFont *font,
  * This face will be kept around until you call
  * pango_fc_font_unlock_face().
  *
- * Returns: the FreeType FT_Face associated with @font.
+ * Return value: the FreeType FT_Face associated with @font.
  *
  * Since: 1.4
  **/
@@ -631,6 +643,8 @@ pango_fc_font_get_glyph (PangoFcFont *font,
  * Returns the index of a glyph suitable for drawing @wc as an
  * unknown character.
  *
+ * Use PANGO_GET_UNKNOWN_GLYPH() instead.
+ *
  * Return value: a glyph index into @font.
  *
  * Since: 1.4
@@ -639,9 +653,10 @@ PangoGlyph
 pango_fc_font_get_unknown_glyph (PangoFcFont *font,
 				 gunichar     wc)
 {
-  g_return_val_if_fail (PANGO_IS_FC_FONT (font), 0);
+  if (font && PANGO_FC_FONT_GET_CLASS (font)->get_unknown_glyph)
+    return PANGO_FC_FONT_GET_CLASS (font)->get_unknown_glyph (font, wc);
 
-  return PANGO_FC_FONT_GET_CLASS (font)->get_unknown_glyph (font, wc);
+  return PANGO_GET_UNKNOWN_GLYPH (wc);
 }
 
 void
@@ -706,7 +721,7 @@ pango_fc_font_kern_glyphs (PangoFcFont      *font,
  *
  * This will return any custom decoder set on this font.
  *
- * Returns: The custom decoder
+ * Return value: The custom decoder
  *
  * Since: 1.6
  **/
@@ -813,7 +828,7 @@ pango_fc_font_get_raw_extents (PangoFcFont    *fcfont,
 
   face = PANGO_FC_FONT_LOCK_FACE (fcfont);
 
-  if (!glyph)
+  if (glyph == PANGO_GLYPH_EMPTY)
     gm = NULL;
   else
     gm = get_per_char (face, load_flags, glyph);
