@@ -27,6 +27,8 @@
 #include "pangoxft-private.h"
 #include "pangofc-private.h"
 
+PangoXftWarningHistory _pango_xft_warning_history = { FALSE };
+
 #define PANGO_XFT_FONT_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), PANGO_TYPE_XFT_FONT, PangoXftFontClass))
 #define PANGO_XFT_IS_FONT_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), PANGO_TYPE_XFT_FONT))
 #define PANGO_XFT_FONT_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), PANGO_TYPE_XFT_FONT, PangoXftFontClass))
@@ -51,8 +53,6 @@ static gboolean   pango_xft_font_real_has_char          (PangoFcFont      *font,
 							 gunichar          wc);
 static guint      pango_xft_font_real_get_glyph         (PangoFcFont      *font,
 							 gunichar          wc);
-static PangoGlyph pango_xft_font_real_get_unknown_glyph (PangoFcFont      *font,
-							 gunichar          wc);
 static void       pango_xft_font_real_shutdown          (PangoFcFont      *font);
 
 static XftFont *xft_font_get_font (PangoFont *font);
@@ -74,7 +74,6 @@ pango_xft_font_class_init (PangoXftFontClass *class)
   fc_font_class->unlock_face = pango_xft_font_real_unlock_face;
   fc_font_class->has_char = pango_xft_font_real_has_char;
   fc_font_class->get_glyph = pango_xft_font_real_get_glyph;
-  fc_font_class->get_unknown_glyph = pango_xft_font_real_get_unknown_glyph;
   fc_font_class->shutdown = pango_xft_font_real_shutdown;
 }
 
@@ -124,7 +123,8 @@ _pango_xft_font_get_mini_font (PangoXftFont *xfont)
 {
   PangoFcFont *fcfont = (PangoFcFont *)xfont;
 
-  g_assert (fcfont->fontmap);
+  if (!fcfont || !fcfont->fontmap)
+    return NULL;
 
   if (!xfont->mini_font)
     {
@@ -153,6 +153,8 @@ _pango_xft_font_get_mini_font (PangoXftFont *xfont)
 	pango_font_description_set_size (desc, new_size);
 
       xfont->mini_font = pango_font_map_load_font (fcfont->fontmap, context, desc);
+      if (!xfont->mini_font)
+        return NULL;
 
       pango_font_description_free (desc);
       g_object_unref (context);
@@ -170,7 +172,7 @@ _pango_xft_font_get_mini_font (PangoXftFont *xfont)
       
       xfont->mini_width = PANGO_SCALE * width;
       xfont->mini_height = PANGO_SCALE * height;
-      xfont->mini_pad = PANGO_SCALE * MAX (height / 10, 1);
+      xfont->mini_pad = PANGO_SCALE * MAX ((int)(2.2 * height + 27) / 28, 1);
     }
 
   return xfont->mini_font;
@@ -209,7 +211,7 @@ get_glyph_extents_missing (PangoXftFont    *xfont,
   PangoFont *font = PANGO_FONT (xfont);
   XftFont *xft_font = xft_font_get_font (font);
   
-  gint cols = (glyph & ~PANGO_XFT_UNKNOWN_FLAG) > 0xffff ? 3 : 2;
+  gint cols = (glyph & ~PANGO_GLYPH_UNKNOWN_FLAG) > 0xffff ? 3 : 2;
   
   _pango_xft_font_get_mini_font (xfont);
   
@@ -268,6 +270,12 @@ typedef struct
   PangoRectangle logical_rect;
 } Extents;
 
+static void
+extents_free (Extents *ext)
+{
+  g_slice_free (Extents, ext);
+}
+
 static void
 get_glyph_extents_raw (PangoXftFont     *xfont,
 		       PangoGlyph        glyph,
@@ -278,14 +286,14 @@ get_glyph_extents_raw (PangoXftFont     *xfont,
 
   if (!xfont->glyph_info)
     xfont->glyph_info = g_hash_table_new_full (NULL, NULL,
-					       NULL, (GDestroyNotify)g_free);
+					       NULL, (GDestroyNotify)extents_free);
 
   extents = g_hash_table_lookup (xfont->glyph_info,
 				 GUINT_TO_POINTER (glyph));
 
   if (!extents)
     {
-      extents = g_new (Extents, 1);
+      extents = g_slice_new (Extents);
      
       pango_fc_font_get_raw_extents (PANGO_FC_FONT (xfont),
 				     FT_LOAD_NO_BITMAP | FT_LOAD_NO_HINTING,
@@ -317,39 +325,26 @@ pango_xft_font_get_glyph_extents (PangoFont        *font,
   if (!fcfont->fontmap)		/* Display closed */
     goto fallback;
 
-  if (glyph == (PangoGlyph)-1)
-    glyph = 0;
-
-  if (glyph & PANGO_XFT_UNKNOWN_FLAG)
+  if (glyph == PANGO_GLYPH_EMPTY)
+    {
+    fallback:
+      if (ink_rect)
+	ink_rect->x = ink_rect->width = ink_rect->y = ink_rect->height = 0;
+      if (logical_rect)
+	logical_rect->x = logical_rect->width = logical_rect->y = logical_rect->height = 0;
+      return;
+    }
+  if (glyph & PANGO_GLYPH_UNKNOWN_FLAG)
     {
       get_glyph_extents_missing (xfont, glyph, ink_rect, logical_rect);
     }
-  else if (glyph)
+  else
     {
       if (!fcfont->is_transformed)
 	get_glyph_extents_xft (fcfont, glyph, ink_rect, logical_rect);
       else
 	get_glyph_extents_raw (xfont, glyph, ink_rect, logical_rect);
     }
-  else
-    {
-    fallback:
-      
-      if (ink_rect)
-	{
-	  ink_rect->x = 0;
-	  ink_rect->width = 0;
-	  ink_rect->y = 0;
-	  ink_rect->height = 0;
-	}
-      if (logical_rect)
-	{
-	  logical_rect->x = 0;
-	  logical_rect->width = 0;
-	  logical_rect->y = 0;
-	  logical_rect->height = 0;
-	}
-    }
 }
 
 static void
@@ -443,17 +438,10 @@ pango_xft_font_real_get_glyph (PangoFcFont *font,
 			       gunichar     wc)
 {
   XftFont *xft_font = xft_font_get_font ((PangoFont *)font);
-
+  
   return XftCharIndex (NULL, xft_font, wc);
 }
 
-static PangoGlyph
-pango_xft_font_real_get_unknown_glyph (PangoFcFont *font,
-				       gunichar     wc)
-{
-  return wc | PANGO_XFT_UNKNOWN_FLAG;
-}
-
 static void
 pango_xft_font_real_shutdown (PangoFcFont *fcfont)
 {
@@ -475,12 +463,20 @@ pango_xft_font_real_shutdown (PangoFcFont *fcfont)
  *
  * Returns the XftFont of a font.
  *
- * Returns: the XftFont associated to @font.
+ * Return value: the XftFont associated to @font.
  **/
 XftFont *
 pango_xft_font_get_font (PangoFont *font)
 {
-  g_return_val_if_fail (PANGO_XFT_IS_FONT (font), NULL);
+  if (G_UNLIKELY (!PANGO_XFT_IS_FONT (font)))
+    {
+      if (!_pango_xft_warning_history.get_font)
+        {
+	  _pango_xft_warning_history.get_font = TRUE;
+	  g_warning ("pango_xft_font_get_font called with bad font, expect ugly output");
+	}
+      return NULL;
+    }
   
   return xft_font_get_font (font);
 }
@@ -491,7 +487,7 @@ pango_xft_font_get_font (PangoFont *font)
  *
  * Returns the X display of the XftFont of a font.
  *
- * Returns: the X display of the XftFont associated to @font.
+ * Return value: the X display of the XftFont associated to @font.
  **/
 Display *
 pango_xft_font_get_display (PangoFont *font)
@@ -515,7 +511,7 @@ pango_xft_font_get_display (PangoFont *font)
  * Returns the index of a glyph suitable for drawing @wc as an
  * unknown character.
  *
- * Use pango_fc_font_get_unknown_glyph() instead.
+ * Use PANGO_GET_UNKNOWN_GLYPH() instead.
  *
  * Return value: a glyph index into @font.
  **/
@@ -523,7 +519,7 @@ PangoGlyph
 pango_xft_font_get_unknown_glyph (PangoFont *font,
 				  gunichar   wc)
 {
-  g_return_val_if_fail (PANGO_XFT_IS_FONT (font), -1);
+  g_return_val_if_fail (PANGO_XFT_IS_FONT (font), PANGO_GLYPH_EMPTY);
 
   return pango_fc_font_get_unknown_glyph (PANGO_FC_FONT (font), wc);
 }
@@ -538,7 +534,7 @@ pango_xft_font_get_unknown_glyph (PangoFont *font,
  *
  * Use pango_fc_font_lock_face() instead.
  *
- * Returns: the FreeType FT_Face associated with @font.
+ * Return value: the FreeType FT_Face associated with @font.
  *
  * Since: 1.2
  **/
